resource "aws_appsync_resolver" "User_friends" {
  api_id = aws_appsync_graphql_api.appsync.id
  type   = "User"
  field  = "friends"
  runtime {
    name            = "APPSYNC_JS"
    runtime_version = "1.0.0"
  }
  code = <<EOF
export function request(ctx) {
	return {};
}
export function response(ctx) {
	if (ctx.stash.earlyReturnNull) {
		return null;
	}else {
		return ctx.result;
	}
}
EOF
  kind = "PIPELINE"
  pipeline_config {
    functions = [
      aws_appsync_function.User_friends_1.function_id,
      aws_appsync_function.User_friends_2.function_id,
      aws_appsync_function.User_friends_3.function_id,
    ]
  }
}
resource "aws_appsync_function" "User_friends_1" {
  api_id      = aws_appsync_graphql_api.appsync.id
  data_source = aws_appsync_datasource.ddb_friend.name
  name        = "User_friends_1"
  runtime {
    name            = "APPSYNC_JS"
    runtime_version = "1.0.0"
  }
  code = <<EOF
import {util, runtime} from "@aws-appsync/utils";
export function request(ctx) {
	// skip friend check for same user
	if (util.authType() !== "User Pool Authorization" || ctx.identity.sub === ctx.source.id) {
		return runtime.earlyReturn({});
	}
	return {
		version : "2018-05-29",
		operation : "GetItem",
		key: {
			userId1: {S: ctx.identity.sub},
			userId2: {S: ctx.source.id},
		}
	};
}
export function response(ctx) {
	if (ctx.error) {
		return util.error(ctx.error.message, ctx.error.type);
	}
	if(!ctx.result) {
		ctx.stash.earlyReturnNull = true;
		runtime.earlyReturn("null");
	}
	return ctx.prev.result;
}
EOF
}
resource "aws_appsync_function" "User_friends_2" {
  api_id      = aws_appsync_graphql_api.appsync.id
  data_source = aws_appsync_datasource.ddb_friend.name
  name        = "User_friends_2"
  runtime {
    name            = "APPSYNC_JS"
    runtime_version = "1.0.0"
  }
  code = <<EOF
import {util, runtime} from "@aws-appsync/utils";
export function request(ctx) {
	if (ctx.stash.earlyReturnNull) {
		runtime.earlyReturn("null");
	}
	return {
		version : "2018-05-29",
		operation : "Query",
		query: {
			expression: "#userId1 = :userId1",
			expressionNames: {
				"#userId1": "userId1",
			},
			expressionValues: {
				":userId1": {S: ctx.source.id},
			},
		},
		nextToken: ctx.args.nextToken,
		limit: 4,
	};
}
export function response(ctx) {
	if (ctx.error) {
		return util.error(ctx.error.message, ctx.error.type);
	}
	return {
		userIds: ctx.result.items.map(({userId2}) => userId2),
		nextToken: ctx.result.nextToken,
	};
}
EOF
}
resource "aws_appsync_function" "User_friends_3" {
  api_id      = aws_appsync_graphql_api.appsync.id
  data_source = aws_appsync_datasource.ddb_friend.name
  name        = "User_friends_3"
  runtime {
    name            = "APPSYNC_JS"
    runtime_version = "1.0.0"
  }
  code = <<EOF
import {util, runtime} from "@aws-appsync/utils";

const tableName = "${aws_dynamodb_table.user.name}";
export function request(ctx) {
	if (ctx.stash.earlyReturnNull) {
		runtime.earlyReturn("null");
	}
	if (ctx.prev.result.userIds.length === 0) {
		runtime.earlyReturn({users: [], nextToken: null});
	}
	return {
		version : "2018-05-29",
		operation : "BatchGetItem",
		tables: {
			[tableName]: {
				keys: ctx.prev.result.userIds.map((userId) => ({
					id: {S: userId},
				})),
				consistentRead: true,
			},
		},
	};
}
export function response(ctx) {
	if (ctx.error) {
		return util.error(ctx.error.message, ctx.error.type);
	}
	if(ctx.result.unprocessedKeys[tableName].length > 0) {
		return util.error("Batch get failed for some items");
	}
	return {
		users: ctx.result.data[tableName],
		nextToken: ctx.prev.result.nextToken,
	};
}
EOF
}

