import {setTimeout} from "node:timers/promises";
import {SignatureV4} from "@aws-sdk/signature-v4";
import {HttpRequest} from "@aws-sdk/protocol-http";
import {defaultProvider} from "@aws-sdk/credential-provider-node";
import {URL} from "url";
import {Hash} from "@aws-sdk/hash-node";

const {APIURL, APIREGION} = process.env;

export const getIAMAuthRequest = async (APIURL, region, data) => {
	const url = new URL(APIURL);
	const httpRequest = new HttpRequest({
		body: JSON.stringify(data),
		headers: {
			"content-type": "application/json; charset=UTF-8",
			"content-encoding": "amz-1.0",
			"accept": "application/json, text/javascript",
			host: url.hostname,
		},
		hostname: url.hostname,
		method: "POST",
		path: url.pathname,
		protocol: url.protocol,
		query: {},
	});
	
	const signer = new SignatureV4({
		credentials: defaultProvider(),
		service: "appsync",
		region: region,
		sha256: Hash.bind(null, "sha256"),
	});
	const req = await signer.sign(httpRequest);
	return req;
};

const sendQuery = async (query, operationName, variables) => {
	const req = await getIAMAuthRequest(APIURL, APIREGION, {query, operationName, variables});
	const res = await fetch(`${req.protocol}//${req.hostname}${req.path}`, {
		method: req.method,
		body: req.body,
		headers: req.headers,
	});

	if (!res.ok) {
		throw new Error("Failed");
	}
	const resJson = await res.json();
	if (resJson.errors) {
		throw new Error(resJson);
	}
	return resJson;
};

const getPaginatedResults = async (fn) => {
	const EMPTY = Symbol("empty");
	const res = [];
	for await (const lf of (async function*() {
		let NextMarker = EMPTY;
		while (NextMarker || NextMarker === EMPTY) {
			const {marker, results} = await fn(NextMarker !== EMPTY ? NextMarker : undefined);

			yield* results;
			NextMarker = marker;
		}
	})()) {
		res.push(lf);
	}

	return res;
};

const randomComment = () => {
	const possibilities = [
		"Comment Ipsum!",
		"Hell yeah!",
		"Great!",
		"Good!",
		"Yeah!",
		"Commenting!",
		"Hmmm....",
		"Interesting!",
	];
	return possibilities[Math.floor(Math.random() * possibilities.length)];
};

const randomPost = () => {
	const possibilities = [
		"Lorem Ipsum",
		"Dolor set ameth",
		"Hello World!",
		"Something",
		"Something interesting",
		"What's up?",
		"All good!",
	];
	return possibilities[Math.floor(Math.random() * possibilities.length)];
};

const updateData = async () => {
	const userIds = JSON.parse(process.env.USERIDS);
	await Promise.all(userIds.map(async (userId) => {
		const myPostsQuery = await sendQuery(`
query MyQuery($userId: ID!) {
  user(id: $userId) {
		name
    posts(limit: 20) {
      posts {
        id
      }
    }
  }
}
			`, "MyQuery", {userId});
		// comment on own posts
		const posts = myPostsQuery.data.user.posts.posts;
		await Promise.all(posts.map(async ({id}, index) => {
			if (Math.random() < (0.05 * (1 / 2 ** index))) {
				console.log(`${myPostsQuery.data.user.name}: Adding a new comment to a user's own post`);
				await sendQuery(`
mutation MyMutation($userId: ID!, $postId: ID!, $text: String!) {
  addComment(userId: $userId, postId: $postId, text: $text) {
    id
  }
}
			`, "MyMutation", {userId, postId: id, text: randomComment()});
			}
		}));
		await setTimeout(2000);
		// add new post
		if (Math.random() < 0.1) {
			console.log(`${myPostsQuery.data.user.name}: Adding a new post`);
			await sendQuery(`
mutation MyMutation($userId: ID!, $text: String!) {
  createPost(userId: $userId, text: $text) {
    id
  }
}
			`, "MyMutation", {userId, text: randomPost()});
		}
		await setTimeout(2000);
		// write comment for friends
		const friendsPostsQuery = await sendQuery(`
query MyQuery($userId: ID!) {
  user(id: $userId) {
    friends {
      users {
				name
        posts(limit: 10) {
          posts {
            id
          }
        }
      }
    }
  }
}
			`, "MyQuery", {userId});
		await Promise.all(friendsPostsQuery.data.user.friends.users.map(async (friend) => {
			await Promise.all(friend.posts.posts.map(async ({id}, index) => {
				if (Math.random() < (0.05 * (1 / 2 ** index))) {
					await setTimeout(100);
					console.log(`${myPostsQuery.data.user.name} => ${friend.name}: Adding a comment to another user's post`);
					await sendQuery(`
mutation MyMutation($userId: ID!, $postId: ID!, $text: String!) {
  addComment(userId: $userId, postId: $postId, text: $text) {
    id
  }
}
			`, "MyMutation", {userId, postId: id, text: randomComment()});
				}
			}));
		}));
	}));
};

export const handler = async (event, context) => {
	await updateData();
	while(context.getRemainingTimeInMillis() > 15000) {
		console.log(context.getRemainingTimeInMillis());
		await setTimeout(10000);
		await updateData();
	}
};


