import usePromise from "react-use-promise";
import htm from "htm";
import React, {useState, useEffect} from "react";
import {produce, current} from "immer";
import {makePaginatedFetcher, formatDate, sendAppSyncQuery, persistentSubscription, connection} from "./utils.mjs";
import {APIURL} from "./utils.mjs";

const html = htm.bind(React.createElement);

export const App = ({loggedin, getAccessToken, redirectToLogin, redirectToLogout}) => {
	const query = sendAppSyncQuery(getAccessToken);

	const displayUserId = location.pathname.match(/\/(?<userid>.+)$/)?.groups?.userid;

	const [data, setData] = useState(undefined);

	const [, loadError, loadState] = usePromise(async () => {
		if(loggedin) {
			const friendsSubquery = `
				friends(nextToken: $friendsNextToken) {
					nextToken
					users {
						avatar
						id
						name
					}
				}
			`;

			const fetchFriends = makePaginatedFetcher(async (nextToken) => {
				return (await query(`
					query MyQuery($friendsNextToken: String${displayUserId ? ", $id: ID!" : ""}) {
						user: ${displayUserId ? "user(id: $id)" : "currentUser"} {
							${friendsSubquery}
						}
					}
				`, {friendsNextToken: nextToken, id: displayUserId})).user.friends;
			}, (user) => user, {items: "users", nextToken: "nextToken"})

			const currentUserSubquery = `
currentUser {
	id
	name
	avatar
}
			`;
			const userSubquery = `
id
name
avatar
posts(limit: 3) {
	nextToken
	posts {
		date
		id
		text
		comments(limit: 3) {
			nextToken
			comments {
				date
				id
				text
				author {
					avatar
					id
					name
				}
			}
		}
	}
}
${friendsSubquery}
			`;
			const firstData = displayUserId ? await query(`
query MyQuery($friendsNextToken: String, $id: ID!) {
	${currentUserSubquery}
	user(id: $id) {
		${userSubquery}
  }
}
				`, {id: displayUserId}) : await query(`
query MyQuery($friendsNextToken: String) {
	${currentUserSubquery}
	user: currentUser {
		${userSubquery}
  }
}
					`, {});

			const data = {
				...firstData,
				user: {
					...firstData.user,
					friends: firstData.user.friends === null ? firstData.user.friends : await fetchFriends()(firstData.user.friends),
				}
			};
			setData(data);
		}
	}, []);

	useEffect(() => {
		if (data === undefined) {
			return () => {};
		}else {
			const query = `
subscription MySubscription($userId: ID!) {
  post(userId: $userId) {
    post {
      date
      id
      text
    }
  }
}
			`;
			const subscription = persistentSubscription(connection)({
				getAuthorizationHeaders: async () => ({host: new URL(APIURL).host, Authorization: await getAccessToken()}),
			})(query, {userId: data.user.id})
			.subscribe({
				next: (e) => {
					setData(produce((data) => {
						if (!data.user.posts.posts.find(({id}) => id === e.data.post.post.id)) {
							data.user.posts.posts.unshift({...e.data.post.post, comments: {nextToken: null, comments: []}});
						}
					}));
				},
				error: (e) => console.log("subscription.error", e),
				complete: () => console.log("subscription.complete"),
			});
			return () => {
				subscription?.unsubscribe();
			};
		}
	}, [data?.user.id]);

	const firstPostDate = data?.user.posts?.posts.find((post, index, l) => l.every((comparePost) => new Date(post.date).getTime() <= new Date(comparePost.date).getTime())).date;

	useEffect(() => {
		if (data === undefined) {
			return () => {};
		}else {
			const query = `
subscription MySubscription($postDateStarting: AWSDateTime!, $postUserId: ID!) {
  comment(postDateStarting: $postDateStarting, postUserId: $postUserId) {
    comment {
      date
      id
      text
			author {
				id
			}
			post {
				id
			}
    }
  }
}
			`;
			const subscription = persistentSubscription(connection)({
				getAuthorizationHeaders: async () => ({host: new URL(APIURL).host, Authorization: await getAccessToken()}),
			})(query, {postUserId: data.user.id, postDateStarting: new Date(firstPostDate).toISOString()})
			.subscribe({
				next: (e) => {
					setData(produce((data) => {
						const post = data.user.posts.posts.find(({id}) => id === e.data.comment.comment.post.id);
						if (!post.comments.comments.find(({id}) => id === e.data.comment.comment.id)) {
							const author = [{id: data.user.id, name: data.user.name, avatar: data.user.avatar}, ...data.user.friends.users].find(({id}) => id === e.data.comment.comment.author.id);
							post.comments.comments.unshift({...e.data.comment.comment, author});
						}
					}));
				},
				error: (e) => console.log("subscription.error", e),
				complete: () => console.log("subscription.complete"),
			});
			return () => {
				subscription?.unsubscribe();
			};
		}
	}, [firstPostDate]);


	const writePost = async (event) => {
		event.preventDefault();
		[...event.target.querySelectorAll("*")].forEach((e) => e.disabled = true);
		const text = event.target.querySelector("input").value;

		const res = await query(`
mutation MyQuery($text: String!, $userId: ID!) {
	createPost(text: $text, userId: $userId) {
		date
		id
		text
	}
}
			`, {text, userId: data.currentUser.id});

		setData(produce((data) => {
			data.user.posts.posts.unshift({...res.createPost, comments: {nextToken: null, comments: []}});
		}));

		event.target.querySelector("input").value = "";
		[...event.target.querySelectorAll("*")].forEach((e) => e.disabled = false);
	};

	const writeComment = (postId) => async (event) => {
		event.preventDefault();
		[...event.target.querySelectorAll("*")].forEach((e) => e.disabled = true);
		const text = event.target.querySelector("input").value;

		const res = await query(`
mutation MyQuery($text: String!, $userId: ID!, $postId: ID!) {
	addComment(text: $text, userId: $userId, postId: $postId) {
		date
		id
		text
		author {
			id
			name
			avatar
		}
	}
}
			`, {text, userId: data.currentUser.id, postId});

		setData(produce((data) => {
			const post = data.user.posts.posts.find(({id}) => id === postId);
			post.comments.comments.unshift(res.addComment);
		}));

		event.target.querySelector("input").value = "";
		[...event.target.querySelectorAll("*")].forEach((e) => e.disabled = false);
	};

	const loadMorePosts = async (e) => {
		e.target.disabled = true;
		const res = await query(`
query MyQuery($userId: ID!, $nextToken: String) {
	user(id: $userId) {
    posts(limit: 3, nextToken: $nextToken) {
      nextToken
      posts {
        date
        id
        text
        comments(limit: 3) {
          nextToken
          comments {
            date
            id
            text
            author {
              avatar
              id
              name
            }
          }
        }
      }
    }
  }
}
				`, {userId: data.user.id, nextToken: data.user.posts.nextToken});
		setData(produce((data) => {
			data.user.posts.nextToken = res.user.posts.nextToken;
			data.user.posts.posts = [...data.user.posts.posts, ...res.user.posts.posts];
		}));
		e.target.disabled = false;
	};

	const loadMoreComments = (postId) => async (e) => {
		e.target.disabled = true;
		const res = await query(`
query MyQuery($postId: ID!, $nextToken: String) {
	post(id: $postId) {
		comments(limit: 3, nextToken: $nextToken) {
			nextToken
			comments {
				date
				id
				text
				author {
					avatar
					id
					name
				}
			}
		}
  }
}
				`, {postId, nextToken: data.user.posts.posts.find(({id}) => id === postId).comments.nextToken});
		setData(produce((data) => {
			const post = data.user.posts.posts.find(({id}) => id === postId);
			post.comments.nextToken = res.post.comments.nextToken;
			post.comments.comments = [...post.comments.comments, ...res.post.comments.comments];
		}));
		e.target.disabled = false;
	};

	return html`
		<div class="container">
			<ul class="nav justify-content-end mb-5">
				<li class="nav-item">
					<a class="nav-link">${loggedin ? html`
						<span>Logged in</span>
							${data !== undefined && html`
								<span> as <a href="/">${data.currentUser.name}</a></span>
								<img class="rounded avatar ms-2" src=${`data:image/svg+xml;base64,${btoa(data.currentUser.avatar)}`}/>
							`}
					` : "Not logged in"}</a>
				</li>
				<li class="nav-item align-self-center">
					${loggedin ?
						html`<a class="nav-link" href="#" onClick=${redirectToLogout}>Logout</a>`
						: html`<a class="nav-link" href="#" onClick=${redirectToLogin}>Login</a>`
					}
				</li>
			</ul>
			${!loggedin && html`
				<div>
					<h1>Welcome to this example project!</h1>
					<p>To view the posts and comments <a href="#" onClick=${redirectToLogin}>log in</a> with:
						<ul>
							<li>user // Password.1</li>
						</ul>
					</p>
				</div>
			`}
			${loadError ? html`
				<pre>
					${JSON.stringify(loadError, undefined, 4)}
				</pre>
			` : ""}
			${loadState === "pending" ? html`<div class="text-center my-5"><div class="spinner-border" role="status"/></div>` : ""}
			${data && html`
				<div class="row">
					<div class="col-md-8">
						<${UserPosts} user=${data.user} currentUserId=${data.currentUser.id} writePost=${displayUserId ? undefined : writePost} writeComment=${writeComment} loadMorePosts=${loadMorePosts} loadMoreComments=${loadMoreComments}/>
					</div>
					<div class="col-md-3 offset-md-1 mt-5">
						<div class="h4 text-end">${data.user.name}'s friends</div>
						${data.user.friends === null ? html`
							<span>Not friends</span>
						` : data.user.friends.users.map((friend) => html`
							<p class="mb-4 text-end h4"><a href=${`/${friend.id === data.currentUser.id ? "" : friend.id}`}>${friend.name}</a>
								<img class="rounded avatar ms-3" src=${`data:image/svg+xml;base64,${btoa(friend.avatar)}`}/>
							</p>
						`)}
					</div>
				</div>
			`}
		</div>
	`;
};

const UserPosts = ({user, currentUserId, writePost, writeComment, loadMorePosts, loadMoreComments}) => {
	return html`
		<h2>${user.name}'s posts
			<img class="rounded avatar ms-3" src=${`data:image/svg+xml;base64,${btoa(user.avatar)}`}/>
		</h2>
		${writePost && html`
			<form onSubmit=${writePost}>
				<div class="mb-3">
					<label for="postText" class="form-label">Write a new post</label>
					<input required type="text" class="form-control" id="postText"/>
				</div>
				<button type="submit" class="btn btn-outline-primary">Submit</button>
			</form>
		`}
		${user.posts === null ? html`
			<span>Not friends, can't show posts</span>
		` : user.posts.posts.length === 0 ? html`
			<span>No posts yet</span>
		` : user.posts.posts.map((post) => html`
			<div class="card my-5">
				<div class="card-body">
					<p class="card-text">
						<div class="d-flex justify-content-between mb-3">
							<span class="badge text-bg-info">Post</span>
							<span class="text-muted">
								${formatDate(post.date)}
							</span>
						</div>
						${post.text}
					</p>
					<p class="card-text text-end">By <a href=${`/${user.id === currentUserId ? "" : user.id}`}>${user.name}</a> <img class="rounded avatar" src=${`data:image/svg+xml;base64,${btoa(user.avatar)}`}/></p>
					<form onSubmit=${writeComment(post.id)}>
						<div class="mb-3">
							<label for="postText" class="form-label">Add comment</label>
							<input required type="text" class="form-control" id="commentText"/>
						</div>
						<button type="submit" class="btn btn-outline-primary mb-4">Submit</button>
					</form>
					${post.comments.comments.map((comment) => html`
						<div class="card">
							<div class="card-body">
								<p class="card-text">
									<div class="d-flex justify-content-between mb-3">
										<span class="badge text-bg-light">Comment</span>
										<span class="text-muted">
											${formatDate(comment.date)}
										</span>
									</div>
									${comment.text}
								</p>
								<p class="card-text text-end">By <a href=${`/${comment.author.id === currentUserId ? "" : comment.author.id}`}>${comment.author.name}</a> <img class="rounded avatar" src=${`data:image/svg+xml;base64,${btoa(comment.author.avatar)}`}/></p>
							</div>
						</div>
					`)}
					${post.comments.nextToken && html`
						<button class="btn btn-outline-secondary my-3" onClick=${loadMoreComments(post.id)}>Load more comments</button>
					`}
				</div>
			</div>
		`)}
		${user.posts?.nextToken && html`
			<button class="btn btn-outline-secondary my-3" onClick=${loadMorePosts}>Load more posts</button>
		`}
	`;
}

