provider "aws" {
}

resource "random_id" "id" {
  byte_length = 8
}

resource "aws_iam_role" "appsync" {
  assume_role_policy = <<EOF
{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Action": "sts:AssumeRole",
      "Principal": {
        "Service": "appsync.amazonaws.com"
      },
      "Effect": "Allow"
    }
  ]
}
EOF
}

data "aws_iam_policy_document" "appsync" {
  statement {
    actions = [
      "dynamodb:PutItem",
      "dynamodb:UpdateItem",
      "dynamodb:GetItem",
      "dynamodb:Query",
      "dynamodb:BatchGetItem",
    ]
    resources = [
      aws_dynamodb_table.user.arn,
      aws_dynamodb_table.friend.arn,
      "${aws_dynamodb_table.friend.arn}/*",
      aws_dynamodb_table.post.arn,
      "${aws_dynamodb_table.post.arn}/*",
      aws_dynamodb_table.comment.arn,
      "${aws_dynamodb_table.comment.arn}/*",
    ]
  }
  statement {
    actions = [
      "appsync:GraphQL",
    ]
    resources = [
      "${aws_appsync_graphql_api.appsync.arn}/types/Mutation/fields/notifyPost",
      "${aws_appsync_graphql_api.appsync.arn}/types/Mutation/fields/notifyComment",
    ]
  }
}

resource "aws_iam_role_policy" "appsync" {
  role   = aws_iam_role.appsync.id
  policy = data.aws_iam_policy_document.appsync.json
}
resource "aws_appsync_graphql_api" "appsync" {
  name                = "social-network"
  schema              = file("schema.graphql")
  authentication_type = "AMAZON_COGNITO_USER_POOLS"
  user_pool_config {
    default_action = "ALLOW"
    user_pool_id   = aws_cognito_user_pool.pool.id
  }
  additional_authentication_provider {
    authentication_type = "AWS_IAM"
  }
  log_config {
    cloudwatch_logs_role_arn = aws_iam_role.appsync_logs.arn
    field_log_level          = "ALL"
  }
}

resource "aws_iam_role" "appsync_logs" {
  assume_role_policy = <<POLICY
{
	"Version": "2012-10-17",
	"Statement": [
		{
		"Effect": "Allow",
		"Principal": {
			"Service": "appsync.amazonaws.com"
		},
		"Action": "sts:AssumeRole"
		}
	]
}
POLICY
}

data "aws_iam_policy_document" "appsync_policy" {
  statement {
    actions = [
      "logs:CreateLogStream",
      "logs:PutLogEvents"
    ]
    resources = [
      "arn:aws:logs:*:*:*"
    ]
  }
}
resource "aws_iam_role_policy" "appsync_logs" {
  role   = aws_iam_role.appsync_logs.id
  policy = data.aws_iam_policy_document.appsync_policy.json
}

resource "aws_cloudwatch_log_group" "loggroup" {
  name              = "/aws/appsync/apis/${aws_appsync_graphql_api.appsync.id}"
  retention_in_days = 14
}

resource "aws_appsync_datasource" "ddb_user" {
  api_id           = aws_appsync_graphql_api.appsync.id
  name             = "ddb_user"
  service_role_arn = aws_iam_role.appsync.arn
  type             = "AMAZON_DYNAMODB"
  dynamodb_config {
    table_name = aws_dynamodb_table.user.name
  }
}
resource "aws_appsync_datasource" "ddb_friend" {
  api_id           = aws_appsync_graphql_api.appsync.id
  name             = "ddb_friend"
  service_role_arn = aws_iam_role.appsync.arn
  type             = "AMAZON_DYNAMODB"
  dynamodb_config {
    table_name = aws_dynamodb_table.friend.name
  }
}
resource "aws_appsync_datasource" "ddb_post" {
  api_id           = aws_appsync_graphql_api.appsync.id
  name             = "ddb_post"
  service_role_arn = aws_iam_role.appsync.arn
  type             = "AMAZON_DYNAMODB"
  dynamodb_config {
    table_name = aws_dynamodb_table.post.name
  }
}
resource "aws_appsync_datasource" "ddb_comment" {
  api_id           = aws_appsync_graphql_api.appsync.id
  name             = "ddb_comment"
  service_role_arn = aws_iam_role.appsync.arn
  type             = "AMAZON_DYNAMODB"
  dynamodb_config {
    table_name = aws_dynamodb_table.comment.name
  }
}

resource "aws_appsync_datasource" "none" {
  api_id = aws_appsync_graphql_api.appsync.id
  name   = "none"
  type   = "NONE"
}

resource "aws_appsync_datasource" "notify" {
  api_id           = aws_appsync_graphql_api.appsync.id
  name             = "notify"
  service_role_arn = aws_iam_role.appsync.arn
  type             = "HTTP"
  http_config {
    endpoint = regex("^[^/]+//[^/]+", aws_appsync_graphql_api.appsync.uris["GRAPHQL"])
    authorization_config {
      aws_iam_config {
        signing_region       = data.aws_region.current.name
        signing_service_name = "appsync"
      }
    }
  }
}
